@@ -12,11 +12,11 @@ from kafka.conn import DEFAULT_SOCKET_TIMEOUT_SECONDS
 
 from .utils import ExponentialSleep, IntervalTracker
 from .const import FETCH_BUFFER_SIZE_BYTES, MAX_FETCH_BUFFER_SIZE_BYTES, GET_MESSAGE_TIMEOUT
-from .offset import SimpleOffsetTracker, WindowedOffsetTracker
 from .message import SourceMessage
 from .kafka_utils import get_kafka_offset_range
 
-class TaskCode(object):
+
+class Task(object):
     def __init__(self, name, code):
         self.name = name
         self.code = code
@@ -55,65 +55,65 @@ class TaskCode(object):
         return self.code.window(state)
 
 
-class Task(object):
+class TaskInstance(object):
 
-    def __init__(self, kafka, code, state, offset_store, config):
+    def __init__(self, kafka, task, partition_id, state_mgr, config):
         self.kafka = kafka
-        self.code = code
+        self.task = task
+        self.partition_id = partition_id
         self.producer = KeyedProducer(self.kafka)
-        self.state = state
-        self.offset_store = offset_store
+        self.state_mgr = state_mgr
         self.last_window_time = time.time()
         self.logger = logging.getLogger('pymza.task.{0}'.format(self.name))
         self.queue = Queue(maxsize=1000)
         self.last_commit = time.time()
         self.config = config
 
-        commit_interval = 5
+        self.state = self.state_mgr.state
+
+        commit_interval = 5*60
         try:
             commit_interval = int(config.get('commit_interval'))
         except (TypeError, ValueError):
             pass
         self.commit_time = IntervalTracker(commit_interval)
 
-        window_interval = self.code.window_interval
+        window_interval = self.task.window_interval
         try:
             window_interval = int(config.get('window_interval'))
         except (TypeError, ValueError):
             pass
 
-        self.result_topics = set(self.code.result_topics)
+        self.result_topics = set(self.task.result_topics)
 
-        if self.code.is_windowed:
+        self.offsets = self.state_mgr.offsets
+        if self.task.is_windowed:
             self.window_time = IntervalTracker(window_interval)
-            self.offsets = WindowedOffsetTracker()
         else:
             self.window_time = None
-            self.offsets = SimpleOffsetTracker()
 
     @property
     def name(self):
-        return self.code.name
+        return self.task.name
 
-    @property
-    def source_topics(self):
-        return self.code.source_topics
+    def __repr__(self):
+        return 'TaskInstance({0}, {1})'.format(self.name, self.partition_id)
 
     def run(self):
-        self.code.init(self.config)
-        self.offset_store.load(self.offsets)
+        self.task.init(self.config)
 
         consumers = []
-        for topic in self.source_topics:
+        for topic in self.task.source_topics:
             consumer = SimpleConsumer(self.kafka,
-                                      self.name, topic, auto_commit=False,
+                                      self.name, topic, 
+                                      partitions=[self.partition_id],
+                                      auto_commit=False,
                                       buffer_size=FETCH_BUFFER_SIZE_BYTES,
                                       max_buffer_size=MAX_FETCH_BUFFER_SIZE_BYTES)
 
-            partition_ids = self.kafka.get_partition_ids_for_topic(topic)
-            server_offsets = get_kafka_offset_range(self.kafka, [(topic, partition_id) for partition_id in partition_ids])
+            server_offsets = get_kafka_offset_range(self.kafka, [(topic, self.partition_id)])
             for topic, partition_id, server_min, server_max in server_offsets:
-                our_offset = self.offsets.get(topic, partition_id)
+                our_offset = self.offsets.get(topic)
                 if our_offset > server_max:
                     raise RuntimeError('Stored offset for topic {0} and partition {1} ({2}) is larger '
                         'than maximal offset on server ({3}). This could be caused by state not synchronized '
@@ -122,16 +122,13 @@ class Task(object):
                     self.logger.warn('Our offset for topic {0} and partition {1} ({2}) is smaller than'
                     ' minimal offset on server ({3}). This could be caused by log compaction. Updating '
                     'our offset to server minimal offset.'.format(topic, partition_id, our_offset, server_min))
-                    self.offsets.force_set(topic, partition_id, server_min)
+                    self.offsets.force_set(topic, server_min)
 
             # set consumer offsets to stored ones
-            for partition in partition_ids:
-                consumer.offsets[partition] = self.offsets.get(topic, partition)
+            consumer.offsets[self.partition_id] = self.offsets.get(topic)
             consumer.fetch_offsets = consumer.offsets.copy()
 
             consumers.append(consumer)
-            self.logger.debug('Initial state: {0} {1} {2}'.format(self.name,
-                                                                  topic, consumer.offsets))
 
         # spawn consumer threads
         for consumer in consumers:
@@ -140,27 +137,33 @@ class Task(object):
 
         while True:
             try:
-                task, consumer, partition, message_envelope = self.queue.get(block=True, timeout=0.5)
+                task, consumer, partition, message_envelope = self.queue.get(block=True, timeout=1)
             except Empty:
                 self.maybe_window()
                 self.maybe_commit()
                 continue
 
             message = SourceMessage(consumer.topic, json.loads(message_envelope.message.value))
-            result = self.code.process(message, self.state)
+            result = self.task.process(message, self.state)
             self.handle_task_result(result)
 
-            self.offsets.set(consumer.topic, partition, message_envelope.offset+1)
+            self.offsets.set(consumer.topic, message_envelope.offset+1)
+
+            # for windowed tasks offsets commited only when `window` called
+            if not self.task.is_windowed:
+                self.offsets.apply_new_offsets()
 
             self.maybe_commit()
 
+            gevent.sleep(0)  # to give chance for other greenlets to run
+
     def maybe_window(self):
         if self.window_time:
-            result = self.code.window(self.state)
+            result = self.task.window(self.state)
             self.handle_task_result(result)
 
             self.window_time.reset()
-            self.offsets.window()
+            self.offsets.apply_new_offsets()
 
     def maybe_commit(self):
         if self.commit_time:
@@ -181,15 +184,8 @@ class Task(object):
         # commit offsets
         if self.offsets.is_modified:
             self.logger.debug('Commiting offsets...')
-            self.offset_store.save(self.offsets)
-            self.offsets.commit()
+            self.state_mgr.commit_offsets()
 
-            # reqs = [kafka.common.OffsetCommitRequest(topic, partition, offset+1, None) for (topic, partition), offset in self.offsets]
-            # resps = self.kafka.send_offset_commit_request(self.name, reqs)
-            # for resp in resps:
-            #     kafka.common.check_error(resp)
-
-            # self.offsets.commit()
 
     def handle_task_result(self, results):
         if results is None:
@@ -205,8 +201,8 @@ class Task(object):
                     raise
                 if key is None:
                     self.logger.error('Task yielded data with no key. Message not sent.')
-                try:
-                    self.producer.send(topic.encode('utf-8'), key.encode('utf-8'), json.dumps(value))
+                try:                    
+                    self.producer.send(ensure_str(topic), ensure_str(key), json.dumps(value))
                 except Exception:
                     self.logger.exception('Failed to send message to {0}:{1}'.format(topic, key))
                     raise
@@ -231,3 +227,9 @@ def _reader_thread(task, consumer, queue):
                 task.logger.exception('Error while reading messages for {0}/{1}. Sleeping for {2}.'.format(
                     task.name, consumer.topic, sleep.current))
             sleep.sleep()
+
+
+def ensure_str(s):
+    if isinstance(s, unicode):
+        return s.encode('utf-8')
+    return s
\ No newline at end of file
